from threading import Lock


class SafeImage:
    def __init__(self):
        self.lock = Lock()
        self.image_container = []

    def set(self, img):
        self.lock.acquire()
        print("setb")
        if len(self.image_container) != 0:
            self.image_container.pop()
        self.image_container.append(img)
        self.lock.release()

    def get(self):
        if len(self.image_container) != 0:
            self.lock.acquire()
            val = None
            try:
                val = self.image_container.pop()
            finally:
                self.lock.release()
                return val



from scvf import constants


def pipeline_loop(locked_image, pipeline_manager, settings, output_consumer, running):
    while running():
        if settings.is_updated():
            pipeline_manager.set_pipeline(settings.get().get(constants.pipeline_name_settings_key, ''))
        img = locked_image.get()
        if img is not None:
            pipeline_manager.process(img)
            output_consumer(pipeline_manager.get_output())



from scvf import constants


def camera_loop(locked_image, camera_manager, settings, running):
    while running():
        if settings.is_updated():
            camera_manager.set_exposure(settings.get().get(constants.exposure_settings_key, ""))
            camera_manager.set_camera(settings.get().get(constants.camera_id_settings_key, ""))

        img = camera_manager.get_image()
        if img is not None:
            locked_image.set(img)
    camera_manager.release()


class Settings:
    def __init__(self):
        self.__values = dict()
        self.__isUpdated = False

    def update(self, **values):
        for key in values.keys():
            if self.__values.get(key, "") != values.get(key, ""):
                self.__isUpdated = True
                self.__values[key] = values[key]

    def get(self):
        self.__isUpdated = False
        return self.__values.copy()

    def is_updated(self):
        return self.__isUpdated



from threading import Thread

from scvf.loops import camera_loop, pipeline_loop, settings_callback
from scvf.util import PipelineManager, SafeImage, CameraManager, Settings


running = False


def start(pipelines, camera_port=0, output_consumer=lambda: None, settings_supplier=lambda x: None):
    global running
    img = SafeImage()
    pipeline_settings = Settings()
    camera_settings = Settings()

    pipeline_manager = PipelineManager(pipelines)
    camera_manager = CameraManager(camera_port)

    pipeline_thread = Thread(target=pipeline_loop, args=(img, pipeline_manager, pipeline_settings,
                                                         output_consumer, lambda: running))
    camera_thread = Thread(target=camera_loop, args=(img, camera_manager, camera_settings, lambda: running))
    running = True
    settings_supplier(settings_callback(camera_settings, pipeline_settings))
    pipeline_thread.start()
    camera_thread.start()


def end():
    global running
    running = False



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="scvf",
    version="0.0.26",
    author="Spikes#2212",
    author_email="spikesno2212@gmail.com",
    description="a framework that wraps opencv for our needs in FRC",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Spikes-2212-Programming-Guild/SpikesCVFramework",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



import cv2
from networktables import NetworkTables


class NetworkTableIO:
    def __init__(self, ip, name):
        NetworkTables.initialize(ip)
        self.nt = NetworkTables.getTable(name)
        self.contour_count = 2

    def settings_supplier(self, callback):
        def entry_listener(table, key, value, isNew):
            callback(key, value)

        self.nt.addEntryListener(entry_listener)

    def output_consumer(self, output):
        contours = sorted(output, key=cv2.contourArea, reverse=True)
        self.contour_count = max(self.contour_count, len(contours))

        # sends info about all the filtered contours received by the function
        for i, c in enumerate(contours):
            self.nt.putNumber("contourArea{}".format(cv2.contourArea(c)))

            x, y, w, h = cv2.boundingRect(c)

            self.nt.putNumber("width{}".format(w))
            self.nt.putNumber("height{}".format(h))
            self.nt.putNumber("x{}".format(x))
            self.nt.putNumber("y{}".format(y))

            self.nt.putBoolean("isUpdated{}".format(i), True)
            self.nt.putNumber("numberOfContours", len(contours))

        # turning off isUpdated flag for contours that were not updated
        for i in range(len(contours), self.contour_count):
            self.nt.putBoolean("isUpdated{}".format(i), False)
